/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.properties.query;

import org.apache.ws.resource.properties.ResourcePropertySet;

import java.net.URI;

/**
 * The <code>QueryEngine</code> interface is used to map queries on the service data of a service to the appropriate
 * <code>ExpressionEvaluators</code> and then return the result. <code>ExpressionEvaluators</code> can be preconfigured
 * or dynamically added at runtime.
 *
 * @see ExpressionEvaluator
 */
public interface QueryEngine
{
    /**
     * Gets the evaluator currently registered to handle an expression of the specified qualified name (from the top
     * level element of the XML Schema definition of the expression)
     *
     * @return the matching expression evaluator or null if none was found
     */
    ExpressionEvaluator getEvaluator( URI dialect );

    /**
     * Registers a new evaluator that can be used to evaluate queries on a service.
     *
     * @param evaluator implementation of evaluator to be used for evaluating queries specified in its XML Schema
     *                  definition.
     */
    void registerEvaluator( ExpressionEvaluator evaluator );

    /**
     * Registers a query expression evaluator for handling query expressions in
     * specified dialect.
     *
     * @param dialect   a query expression dialect
     * @param evaluator a query expression evaluator
     *
     * @throws UnknownQueryExpressionDialectException if the evaluator does not supoport the specified dialect
     */
    void registerEvaluator( URI dialect,
                            ExpressionEvaluator evaluator )
            throws UnknownQueryExpressionDialectException;

    /**
     * Executes a query against a resource property set. The appropriate
     * ExpressionEvaluator is used for the evalution based on the dialect
     * attribute.
     *
     * @param queryExpr       a query expression
     * @param resourcePropSet the resource properties set to evluate the query expression against
     *
     * @return the result of the evaluation which depends on the expression. The results must be an xml serializable
     *         object in order to be passed back correctly to a remote client. The easiest way of achieving this is to
     *         model it as a Bean, or simply return a <code>SOAPElement</code> or <code>DOM Element</code>. If the
     *         result object returned is null an empty query result is returned.
     *
     * @throws UnknownQueryExpressionDialectException if query dialect is unsupported
     * @throws QueryEvaluationErrorException          if query evaluation fails
     * @throws InvalidQueryExpressionException        if query expression is invalid
     */
    Object executeQuery( QueryExpression queryExpr,
                         ResourcePropertySet resourcePropSet )
            throws UnknownQueryExpressionDialectException,
            QueryEvaluationErrorException,
            InvalidQueryExpressionException;

    /**
     * Evaluates the expression over the specified context object and returns the result.
     * The appropriate ExpressionEvaluator is used for the evaluation based on the dialect
     * attribute.
     *
     * @param queryExpr   a query expression
     * @param evalContext the context to evaluate the query expression against
     *
     * @return the result of the evaluation which depends on the expression. The results must be an xml serializable
     *         object in order to be passed back correctly to a remote client. The easiest way of achieving this is to
     *         model it as a Bean, or simply return a <code>SOAPElement</code> or <code>DOM Element</code>. If the
     *         result object returned is null an empty query result is returned.
     *
     * @throws UnknownQueryExpressionDialectException if query dialect is unsupported
     * @throws QueryEvaluationErrorException          if query evaluation fails
     * @throws InvalidQueryExpressionException        if query expression is invalid
     */
    Object executeQuery( QueryExpression queryExpr,
                         Object evalContext )
            throws UnknownQueryExpressionDialectException,
            QueryEvaluationErrorException,
            InvalidQueryExpressionException;

}