/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util.jndi.tools;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.wsfx.wsrf.jndi.config.MetadataConfigDocument;
import org.apache.wsfx.wsrf.jndi.config.MetadataDocument;
import org.apache.xmlbeans.XmlObject;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceType;
import org.xmlsoap.schemas.ws.x2004.x09.mex.LocationDocument;
import org.xmlsoap.schemas.ws.x2004.x09.mex.MetadataReferenceDocument;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * A config object to represent the MetadataExchange information from the
 * jndi-config.xml config file.
 * <p/>
 * This class will store any "loadable" metadata documents, this will avoid
 * unnecessary loading of files.
 *
 * @author Sal Campana
 */
public class MetadataConfigImpl extends MetadataConfig
{
   private static final Log LOG = LogFactory.getLog( MetadataConfigImpl.class );

    /**
     * Constant used for adding/getting the IDENTIFIER map from the dialect map
     */
   private static final String IDENTIFIER_MAP       = "IDENTIFIER_MAP";
    /**
     * Constant for pulling the Complete list of metatdata from the dialect map.
     */
   private static final String COMPLETE_METADATA_LIST = "COMPLETE_METADATA_LIST";

   /** XmlBean MetadataConfigImpl object */
   MetadataConfigDocument.MetadataConfig m_metadata;

   /**
    * Internal Map for storing metatdata.
    *
    * The map will have the structure of:
    *
    * key=dialect
    * value=Map (dialect-specific Map)
    *
    * The dialect-specific Map will have the structure:
    *
    * Two Entries:
    * key= IDENTIFIER_MAP
    * value= A Map keyed on Identifier keyed-to the metatdata
    *
    * key= COMPLETE_METADATA_LIST
    * value= A List containing ALL metadata given this dialect (this includes all entries from IDENTIFIER_MAP)
    *
    **/
   private Map                           m_metatdataMap = new HashMap(  );

   /**
    * Creates a new {@link MetadataConfigImpl} object.
    *
    * @param metadata DOCUMENT_ME
    */
   public MetadataConfigImpl( MetadataConfigDocument.MetadataConfig metadata )
   {
      super(metadata);
      m_metadata = metadata;
      loadMetadata(  );
   }

   /**
    * Returns all known metadata for all dialects.
    *
    * @return A Map keyed on dialect keyed-to a List of metadata
    */
   public Map getAllMetadata(  )
   {
      Map allMetatdata = new HashMap(  );

      //iterate all the metadata
       Iterator iterator = m_metatdataMap.keySet().iterator();
       while (iterator.hasNext())
       {
           String key = (String) iterator.next();
           Map dialectMap = (Map) m_metatdataMap.get(key);
           List all = (List) dialectMap.get(COMPLETE_METADATA_LIST);
           allMetatdata.put(key, all);
       }

      return allMetatdata;
   }

   /**
    * Returns the metadata given a dialect and an identifier.
    *
    * @param dialect
    * @param identifierUri
    * @return  A Map keyed on dialect keyed-to a List of metadata
    */
   public Map getMetadata( String dialect,
                                String identifierUri )
   {
      Map results = new HashMap(  );
      if ( ( dialect != null ) && !"".equals( dialect ) ) //only get metatdata for specific dialect
      {
         Map dialectMap = (Map) m_metatdataMap.get( dialect );
         if ( dialectMap != null )
         {
            if ( identifierUri != null )
            {
               Map    identifierMap = (Map) dialectMap.get( IDENTIFIER_MAP );
               Object metatdata = identifierMap.get( identifierUri );
               if ( metatdata != null )
               {
                   List list = new ArrayList();
                   list.add(metatdata);
                   results.put( dialect, list );
               }
            }
            else
            {
               List metatdataList = (List) dialectMap.get( COMPLETE_METADATA_LIST );
               results.put( dialect, metatdataList );
            }
         }
      }
      else //get all metatdata
      {
         return getAllMetadata(  );
      }

      return results;
   }

   /**
    * Returns metadata based on dialect
    *
    * @param dialect
    * @return A Map keyed on dialect keyed-to a List of metadata
    */
   public Map getMetadata( String dialect )
   {
      return getMetadata( dialect, null );
   }

   /**
    * Adds the parsed-in metadata to the undelying lookup structures.
    *
    * precondition: The structures have already been setup for the metadata
    *
    * @param dialect
    * @param identifier
    * @param metadata   The "parsed" metadata (could be XmlObject, MetadataReference or Location )
    */
   protected void addToMetadataMap( String    dialect,
                                    String    identifier,
                                    XmlObject metadata )
   {
      Map dialectMap = (Map) m_metatdataMap.get( dialect );
      if ( dialectMap == null )
      {
         addStructureForDialect( dialect );
         dialectMap = (Map) m_metatdataMap.get( dialect );
      }

      List metatadataList = (List) dialectMap.get( COMPLETE_METADATA_LIST );
      metatadataList.add( metadata );

      if ( ( identifier != null ) && !"".equals( identifier ) )
      {
         Map identifierMap = (Map) dialectMap.get( IDENTIFIER_MAP );
         identifierMap.put( identifier, metadata );
      }
   }

   /**
    * Builds the lookup structure given a dialect.
    *
    * @param dialect
    */
   private synchronized void addStructureForDialect( String dialect )
   {
      Map dialectMap = new HashMap(  );
      m_metatdataMap.put( dialect, dialectMap ); //add to metadatamap

      //add parsedMetatdata to the list of metadata for this dialect
      dialectMap.put( COMPLETE_METADATA_LIST,
                      new ArrayList(  ) );
      dialectMap.put( IDENTIFIER_MAP,
                      new HashMap(  ) );
   }

   /**
    * Loads up lookup structures with the metadata.
    *
    * This method initializes a Map for a given dialect if it doesn't already exist.  The Map is keyed using
    * 2 constants for pulling a map of identifiers->metatada AND a List of all metatdata.
    */
   private synchronized void loadMetadata(  )
   {
      MetadataDocument.Metadata[] metadataArray = m_metadata.getMetadataArray(  );
      for ( int i = 0; i < metadataArray.length; i++ )
      {
         MetadataDocument.Metadata metadata       = metadataArray[i];
         String                    dialect        = metadata.getDialect(  );
         String                    identifier     = metadata.getIdentifier(  );
         XmlObject                 parsedMetadata = parseMetadata( metadata );

         if ( parsedMetadata != null )
         {
            //see if we already built a map for this dialect, then just add the metatdata
            if ( m_metatdataMap.containsKey( dialect ) )
            {
               addToMetadataMap( dialect, identifier, parsedMetadata );
            }
            else //build the mapping for this dialect
            {
               addStructureForDialect( dialect );
               addToMetadataMap( dialect, identifier, parsedMetadata );
            }
         }
      }
   }

   /**
    * This method will return either a MetadataReferenceDocument,
    * LocationDocument or an XmlObject which has the metadata document parsed into it.
    * <p/>
    * The operation ALWAYS avoids loading the file if the metadata contains a MetadataReference.
    * The operation ALWAYS avoids loading the file if the metadata contains a Location which is a standard Http URL.
    * <p/>
    * The operation WILL load the file if it is a file or a file URL.
    * The operation WILL load the file if it is located in the classpath.
    * <p/>
    * Note: The "load" scenarios are handled last, and if the documents are un-loadable, null will be returned
    *
    * @param metadata
    * @return The XmlObject metadata or null
    */
   private XmlObject parseMetadata( MetadataDocument.Metadata metadata )
   {
      //check for MetadataReference...if its there, we're finished.
      XmlObject             result           = null;
      EndpointReferenceType metadataReferece = metadata.getMetadataReferece(  );
      if ( metadataReferece != null )
      {
         MetadataReferenceDocument metadataReferenceDocument = MetadataReferenceDocument.Factory.newInstance(  );
         metadataReferenceDocument.setMetadataReference( metadataReferece );
         result = metadataReferenceDocument;
      }
      else //use location instead
      {
         String location = metadata.getLocation(  );

         //determine if its a valid URL
         if ( location != null )
         {
            try
            {
               URL url = new URL( location );

               //if we get here then is it a file url??? if so load it
               if ( location.startsWith( "file" ) )
               {
                  try
                  {
                     result = XmlObject.Factory.parse( new File(url.getFile()) );
                  }
                  catch ( Exception e )
                  {
                     LOG.fatal( "*********Unable to load file from file url: " + url+ " CAUSE:", e );
                  }
               }
               else //if not then return a Location...let them do it....
               {
                  LocationDocument locationDocument = LocationDocument.Factory.newInstance(  );
                  locationDocument.setLocation( location );
                  result = locationDocument;
               }
            }
            catch ( MalformedURLException e )
            {
               //not valid..is it a file withoud url prefix? ...load it
               File f = new File( location );
               if ( f.exists(  ) )
               {
                  try
                  {
                     result = XmlObject.Factory.parse( f );
                  }
                  catch ( Exception e1 )
                  {
                     LOG.fatal( "*********Unable to load file: " + f + " CAUSE:", e );
                  }
               }
               else //check classpath
               {
                  URL resource = this.getClass(  ).getClassLoader(  ).getResource( location );
                  try
                  {
                     result = XmlObject.Factory.parse( resource );
                  }
                  catch ( Exception e1 )
                  {
                     LOG.fatal( "*********Unable to load file: " + resource + " from classpath."+ " CAUSE:", e );
                  }
               }
            }
         }
      }

      return result;
   }
}